/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include "netutil/NetworkUtilityFunctions.h"
#include "support/PortableMacros.h"

int Connect(const char * hostName, int port, const char * debugTitle, bool errorsOnly)
{
   struct hostent * he = gethostbyname(hostName);
   if (he)
   {
      int s = socket(AF_INET, SOCK_STREAM, 0);
      if (s >= 0)
      {
         struct sockaddr_in saAddr;
         memset(&saAddr, 0, sizeof(saAddr));
         saAddr.sin_family = AF_INET;
         saAddr.sin_port   = htons(port);
         memcpy(&saAddr.sin_addr.s_addr, he->h_addr, he->h_length);

         if ((debugTitle)&&(errorsOnly == false)) printf("%s: Connecting to [%s:%i]\n", debugTitle, hostName, port);
         if (connect(s, (struct sockaddr *) &saAddr, sizeof(saAddr)) >= 0)
         {
            if ((debugTitle)&&(errorsOnly == false)) printf("%s: Connected to %s:%i\n\n", debugTitle, hostName, port);
            return s;
         }
         else if (debugTitle) printf("%s: connect() to %s:%i failed!\n", debugTitle, hostName, port);

         closesocket(s);
      }
      else if (debugTitle) printf("%s: socket() failed!\n", debugTitle);
   }
   else if (debugTitle) printf("%s: Couldn't lookup hostname [%s]\n", debugTitle, hostName);

   return -1;
}

int CreateAcceptSocket(int listenPort)
{
   return -1;  // TODO: implement this
}


bool IsLessThan(const struct timeval & t1, const struct timeval & t2)
{
   return (t1.tv_sec == t2.tv_sec) ? (t1.tv_usec < t2.tv_usec) : (t1.tv_sec < t2.tv_sec);
}

void AddTimeVal(struct timeval & addToThis, const struct timeval & addThis)
{
   addToThis.tv_sec  += addThis.tv_sec;
   addToThis.tv_usec += addThis.tv_usec;
   if (addToThis.tv_usec > 1000000L)
   {
      addToThis.tv_sec += addToThis.tv_usec / 1000000L;
      addToThis.tv_usec = addToThis.tv_usec % 1000000L;
   }
}                         

void SubtractTimeVal(struct timeval & subtractFromThis, const struct timeval & subtractThis)
{
   subtractFromThis.tv_sec  -= subtractThis.tv_sec;
   subtractFromThis.tv_usec -= subtractThis.tv_usec;
   if (subtractFromThis.tv_usec < 0L)
   {
      subtractFromThis.tv_sec += (subtractFromThis.tv_usec / 1000000L)-1;
      while(subtractFromThis.tv_usec < 0L) subtractFromThis.tv_usec += 1000000L;
   }
}
