/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include "reflector/AbstractReflectSession.h"
#include "reflector/ReflectServer.h"
#include "iogateway/PortableMessageIOGateway.h"

static uint32 _idCounter = 0L;

AbstractReflectSession ::
AbstractReflectSession() : _socket(-1), _gateway(NULL), _owner(NULL)
{
   sprintf(_idString, "%lu", _idCounter++);  
}

AbstractReflectSession ::
~AbstractReflectSession() 
{
   delete _gateway;
}

const char * 
AbstractReflectSession ::
GetHostName() const 
{
   ASSERT(_owner, "Can't call GetHostName() while not attached to the server");
   return _hostName.Cstr();
}

int
AbstractReflectSession ::
GetClientPort() const 
{
   ASSERT(_owner, "Can't call GetClientPort() while not attached to the server");
   return _port;
}

const char * 
AbstractReflectSession ::
GetSessionIDString() const 
{
   return _idString;
}

PortableMessage & 
AbstractReflectSession ::
GetCentralState() const 
{
   ASSERT(_owner, "Can't call GetCentralState() while not attached to the server");
   return _owner->_centralState;
}

status_t 
AbstractReflectSession ::
AddOutgoingMessage(PortableMessageRef ref) 
{
   ASSERT(_owner, "Can't call AddOutgoingMessage() while not attached to the server");
   return (_gateway) ? _gateway->AddOutgoingMessage(ref) : B_ERROR;
}

AbstractMessageIOGateway * 
AbstractReflectSession ::
CreateGateway(int socket)
{
   TCPSocketDataIO * io = new (nothrow) TCPSocketDataIO(socket, false);
   if (io)
   {
      AbstractMessageIOGateway * gw = new (nothrow) PortableMessageIOGateway(io);
      if (gw) return gw;
      io->ReleaseSocket();  // caller still owns the socket on error
      delete io;
   }
   WARN_OUT_OF_MEMORY;
   return NULL;
}

status_t 
AbstractReflectSession ::
AttachedToServer()
{
   return B_NO_ERROR;
}

const PortableQueue<AbstractReflectSessionRef> & 
AbstractReflectSession ::
GetSessionsList() const
{
   ASSERT(_owner, "Can't call GetSessionsIterator() while not attached to the server");
   return _owner->GetSessionsList(); 
}

status_t 
AbstractReflectSession ::
GetSessionByID(const char * id, AbstractReflectSessionRef & returnRef) const
{
   ASSERT(_owner, "Can't call GetSessionByID() while not attached to the server");
   return _owner->_idLookup.Get(id, returnRef) ? B_NO_ERROR : B_ERROR;
}

status_t
AbstractReflectSession ::
AddNewSession(AbstractReflectSessionRef ref, int socket)
{
   ASSERT(_owner, "Can't call AddNewSession() while not attached to the server");
   return _owner->AddNewSession(ref, socket);
}

void
AbstractReflectSession ::
EndSession()
{
   ASSERT(_owner, "Can't call EndSession() while not attached to the server");
   _owner->_lameDuckSessions.AddTail(this);
}
