/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

//! author="Jeremy Friesner"

#ifndef DUMB_REFLECT_SESSION_H
#define DUMB_REFLECT_SESSION_H

#include "reflector/AbstractReflectSession.h"

//:This is a callback function that may be passed to the ServerProcessLoop() function.
// It creates and returns a new DumbReflectSession object.
AbstractReflectSession * DumbSessionCreationFunc();

//:This class represents a single TCP connection between a muscled server and a client program.  
// This class implements a simple "reflect-all-messages-to-all-clients"
// message forwarding strategy, but may be subclassed to perform more complex message routing
// logic.
class DumbReflectSession : public AbstractReflectSession
{
public:
   DumbReflectSession();

   // Called when a new message is received from our IO gateway.
   virtual void MessageReceivedFromGateway(PortableMessageRef msg);

   // Called when a message is sent to us by another session (possibly this one).  
   virtual void MessageReceivedFromNeighbor(AbstractReflectSession & from, PortableMessageRef msg, void * userData);

   // Set this true to enable self-reflection:  if true, messages
   // send by a given client will be reflected back to that client.
   // Default state is false (i.e. messages only go to everyone else)
   void SetReflectToSelf(bool reflectToSelf);

   // Accessor
   bool GetReflectToSelf() const;

protected:
   virtual const char * GetTypeName() const {return "Dumb Session";}

private:
   bool _reflectToSelf;
};

#endif
