/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

//! author="Jeremy Friesner"

#ifndef STRING_TOKENIZER_H
#define STRING_TOKENIZER_H

#include <new.h>
#include <stdio.h>
#include <string.h>
#include "support/PortableMacros.h"

//:String tokenizer class, similar to Java's java.util.StringTokenizer
class StringTokenizer
{
public:
   // Initializes the StringTokenizer to parse (tokenizeMe), which 
   // should be a string of tokens (e.g. words) separated by any 
   // of the characters specified in (separators)
   StringTokenizer(const char * tokenizeMe, const char * separators = ", \t")
   {
      int tlen = strlen(tokenizeMe);
      int slen = strlen(separators);
 
      char * temp = new (nothrow) char[slen + 1 + tlen + 1];
      if (temp)
      {
         strcpy(temp, separators);
         _seps = temp;
         _next = temp + slen + 1;
         strcpy(_next, tokenizeMe);
      }
      else WARN_OUT_OF_MEMORY;
 
      _alloced = true;
   }

   // This ctor is the same as above, only with this one you allow
   // the StringTokenizer to operator directly on (tokenizeMe) rather
   // than making it allocate a copy first.  (it's a bit more efficient)
   // (okay) is ignored...
   StringTokenizer(bool okay, char * tokenizeMe, const char * separators = ", \t")
   {
      _next = tokenizeMe;
      _seps = separators;
      _alloced = false;
   }

   ~StringTokenizer()
   {
      if (_alloced) delete [] _seps;
   }

   // Returns the next token in the parsed string, or NULL if there 
   // are no more tokens left
   char * GetNextToken()
   {
      if (_seps)
      {
         // Move until first non-sep char
         while((*_next)&&(strchr(_seps, *_next) != NULL)) _next++;
         if (*_next)
         {
            char * ret = _next;

            // Move until next sep-char
            while((*_next)&&(strchr(_seps, *_next) == NULL)) _next++;
            if (*_next) 
            {
               *_next = '\0';
               _next++;
            }
            return ret;
         }
      }
      return NULL;
   }

private:
   bool _alloced;
   const char * _seps;
   char * _next;
};

#endif
